/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
    Author:
    Pawe (Googie) Salawa
    boogie@myslenice.one.pl

    ChangeLog:

        [v1.9]
            * xmms-config is searched while configuring.

        [v1.8]
            * Added %d tago to descriptions. It's replaced by
              any description you set manualy.
            * Removed warning while compilation.
  
        [v1.7]
            * Now XMMS tab in Kadu configuration dialog
              is removed after unload module.

        [v1.6]
            * Provides 'descriptions' property and makes
              conflicts with other module which provides this.

        [v1.5]
            * Updated for Kadu 0.3.8-cvs (04.04.2004)

        [v1.4]
            * XMMS menu item changes while XMMS statuses are enabled/disabled.

        [v1.3]
            * Updated for Kadu 0.3.8-cvs (21 II 2004).
            * Fixed for "friends only" statuses.

        [v1.2]
            * Fixed localizations in config dialog.

        [v1.1]
            * Bugfix: Kadu was crashing while getting new message while
              running Kadu with this module.
            * Now Module can be loaded without XMMS runned, but can't be
              used until it isn't runned.
            * Bugfix: Translations wasn't applied to module. Now they are.

        [v1.0]
            * First public release.
*/

#include <xmms/xmmsctrl.h>
#include <stdio.h>

#include "debug.h"
#include "kadu.h"
#include "message_box.h"
#include "userlist.h"
#include "config_dialog.h"
#include "status.h"

#include "xmms.h"

#define MODULE_XMMS_VERSION 1.10

Xmms *xmms_session;

extern "C" int xmms_init()
{
    xmms_session = new Xmms();

    return 0;
}

extern "C" void xmms_close()
{
    delete xmms_session;
}

// Implementation of Xmms class

Xmms::Xmms()
{
    kdebugf();

    menu = new QPopupMenu();
    popups[0] = menu->insertItem(tr("Put formated string"), this, SLOT(putSongTitle(int)));
    popups[1] = menu->insertItem(tr("Put song title"), this, SLOT(putSongTitle(int)));
    popups[2] = menu->insertItem(tr("Put song file name"), this, SLOT(putSongTitle(int)));

    timer = new QTimer();
    connect(timer, SIGNAL(timeout()), this, SLOT(checkTitle()));
    popups[3] = kadu->mainMenu()->insertItem(tr("Enable XMMS statuses"), this, SLOT(toggleTimer(int)), 0, -1, 0);

    connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
    connect(kadu, SIGNAL(changingStatus(int, QString, bool&)), this, SLOT(checkStatusDesc(int, QString, bool&)));

    ConfigDialog::addTab("XMMS");
    ConfigDialog::addLineEdit("XMMS", "XMMS", QT_TRANSLATE_NOOP("@default", "XMMS message format for chat windows"), "chatString", "XMMS: %t [%c / %l]",
        QT_TRANSLATE_NOOP("@default", "%t - song title, %f - file name, %l - song length (MM:SS),<br>%c - current song position (MM:SS), %p - percents of played song"));
    ConfigDialog::addLineEdit("XMMS", "XMMS", QT_TRANSLATE_NOOP("@default", "XMMS status format"), "statusString", "-=[%t]=-",
        QT_TRANSLATE_NOOP("@default", "%t - song title, %f - file name, %l - song length (MM:SS),<br>%c - current song position (MM:SS), %p - percents of played song"));

    descriptionToUse = "";
    descriptionToSet = "";
    xmmsSets = FALSE;
}

Xmms::~Xmms()
{
    kdebugf();
    ConfigDialog::removeControl("XMMS", "XMMS message format for chat windows", "chatString");
    ConfigDialog::removeControl("XMMS", "XMMS status format", "statusString");
    ConfigDialog::removeTab("XMMS");

    delete menu;
    delete timer;
    kadu->mainMenu()->removeItem(popups[3]);

    disconnect(timer, SIGNAL(timeout()), this, SLOT(checkTitle()));
    disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));

}

void Xmms::chatCreated(const UinsList& ul)
{
    kdebugf();
    Chat* chat = chat_manager->findChatByUins(ul);
    
    QPushButton* xmms = new QPushButton(chat->buttontray);
    xmms->setPixmap(icons_manager.loadIcon("Unmute"));
    xmms->setPopup(menu);
    xmms->show();
    QToolTip::add(xmms, "XMMS");
    
}

void Xmms::putSongTitle(int ident)
{
    kdebugf();
    if (!xmms_remote_is_running(0))
    {
        MessageBox::msg(tr("XMMS isn't running!"));
        return;
    }
    Chat* chat = getCurrentChat();
    QString title;
    int id;
    for ( int i = 0; i < 3; i++ )
    {
        if (popups[i] == ident)
        {
            id = i;
            break;
        }
    }

    switch (id)
    {
        case 0:
            title = parse(config_file.readEntry("XMMS", "chatString"));
            break;
        case 1:
            title = getTitle();
            break;
        case 2:
            title = getFile();
            break;
    }

    int x, y;

    HtmlDocument doc, doc2, doc3;
    chat->edit->getCursorPosition(&y, &x);
    chat->edit->insertAt(title, y, x);
    doc.parseHtml(chat->edit->text());
    for ( int i = 0; i < doc.countElements(); i++ )
    {
        if (i == 7)
        {
            doc2.parseHtml(doc.elementText(i));
            for ( int j = 0; j < doc2.countElements(); j++ )
            {
                if (doc2.isTagElement(j))
                    doc3.addTag(doc2.elementText(j));
                else
                    doc3.addText(doc2.elementText(j));
            }
        } else {
            if (doc.isTagElement(i))
                doc3.addTag(doc.elementText(i));
            else
                doc3.addText(doc.elementText(i));
        }
    }
    chat->edit->setText(doc3.generateHtml());
    chat->edit->moveCursor(QTextEdit::MoveEnd, FALSE);
}

QString Xmms::parse(QString str)
{
    kdebugf();
    uint sl = str.length();
    QString r;

    for ( uint i = 0; i < sl; i++ )
    {
        while (str[i] != '%' && i < sl)
        {
            r += str[i];
            i++;
        }

        if (str[i] == '%')
        {
            i++;
            switch(str[i].latin1())
            {
                case 't':
                {
                    r += getTitle();
                    break;
                }
                case 'f':
                {
                    r += getFile();
                    break;
                }
                case 'l':
                {
                    char ms[10];
                    int lgt = getLength() / 1000, m, s;
                    m = lgt / 60;
                    s = lgt % 60;
                    if (s < 10)
                        sprintf(ms, "%d:0%d", m, s);
                    else
                        sprintf(ms, "%d:%d", m, s);

                    r += ms;
                    break;
                }
                case 'c':
                {
                    char ms[10];
                    int lgt = getCurrentPos() / 1000, m, s;
                    m = lgt / 60;
                    s = lgt % 60;
                    if (s < 10)
                        sprintf(ms, "%d:0%d", m, s);
                    else
                        sprintf(ms, "%d:%d", m, s);

                    r += ms;
                    break;
                }
                case 'p':
                {
                    char tmp[10];
                    int perc = getCurrentPos() / getLength();
                    sprintf(tmp, "%d%%", perc);
                    r += tmp;
                    break;
                }
                case 'd':
                {
                    r += descriptionToUse;
                    break;
                }
                default:
                    r += str[i];
            }
        }
    }
    return r;
}

Chat* Xmms::getCurrentChat()
{
    kdebugf();
    ChatList cs = chat_manager->chats();
    uint i;
    for ( i = 0; i < cs.count(); i++ )
    {
        if (cs[i]->isActiveWindow())
        {
            break;
        }
    }
    if (i == cs.count())
        return 0;

    return cs[i];
}

QString Xmms::getTitle()
{
    kdebugf();
    return xmms_remote_get_playlist_title(0, pos());    
}

QString Xmms::getFile()
{
    kdebugf();
    return xmms_remote_get_playlist_file(0, pos());
}

int Xmms::getLength()
{
    kdebugf();
    return xmms_remote_get_playlist_time(0, pos());
}

int Xmms::getCurrentPos()
{
    kdebugf();
    return xmms_remote_get_output_time(0);
}

int Xmms::pos()
{
    return xmms_remote_get_playlist_pos(0);
}

void Xmms::toggleTimer(int id)
{
    kdebugf();
    if (!xmms_remote_is_running(0))
    {
        MessageBox::msg(tr("XMMS isn't running!"));
        return;
    }
    if (timer->isActive())
    {
        kadu->mainMenu()->changeItem(popups[3], tr("Enable XMMS statuses"));
        timer->stop();
    }
    else
    {
        kadu->mainMenu()->changeItem(popups[3], tr("Disable XMMS statuses"));
        timer->start(1000);
    }
}

void Xmms::checkTitle()
{
    int status = getActualStatus(), act;
    switch (GG_S(status))
    {
        case GG_STATUS_AVAIL:
        case GG_STATUS_BUSY:
        case GG_STATUS_INVISIBLE:
            act = 1;
            break;
        case GG_STATUS_AVAIL_DESCR:
        case GG_STATUS_BUSY_DESCR:
        case GG_STATUS_INVISIBLE_DESCR:
            act = 2;
            break;
        default:
            act = 0;
    }
    if (act)
    {
        QString title = xmms_remote_get_playlist_title(0, pos());
        if (title != currentTitle || act == 1)
        {
            currentTitle = title;
            if (act == 1)
                status += 2;

            own_description = parse(config_file.readEntry("XMMS", "statusString"));
            xmmsSets = TRUE;
            kadu->setStatus(status);
            xmmsSets = FALSE;
        }
    }
}

void Xmms::checkStatusDesc(int status, QString desc, bool &stop)
{
    QString str = config_file.readEntry("XMMS", "statusString");
    if (str.find("%d") > -1 && desc != "" && descriptionToSet != desc && !xmmsSets)
    {
        stop = TRUE;
        descriptionToUse = desc;
        own_description = parse(str);
        descriptionToSet = own_description;
        kadu->setStatus(status);
    }
}
